#if (defined(_WIN32) || defined(_WIN64))
/**
 * Copyright (C) 2003-2024, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 *
 * @file fs_3dpdf.h
 * @brief Header file for 3D PDF document related classes and methods.
 */

#ifndef FS_3DPDF_H_
#define FS_3DPDF_H_
#include "common/fs_common.h"
#include "pdf/fs_pdfdoc.h"
#include "pdf/annots/fs_annot.h"

/**
 * @brief Foxit namespace.
 *
 * @details All the definitions and classes of Foxit PDF SDK are defined within this namespace.
 */
namespace foxit {
/** @brief Add-on namespace. */
namespace addon {
/** @brief 3D PDF namespace. */
namespace pdf3d {
/**
 * This class represents a 3D annotation instance.
 *
 * @see @link PDF3DContext @endlink
 */
class PDF3DAnnotInstance FS_FINAL : public Base{
 public:
  /**
  * @brief Enumeration for the 3D controller tool.
  *
  * @details Values of this enumeration should be used alone.
  */
  typedef enum _PDF3DController {
    /** @brief rotate. */
    e_ControllerRotate = 0,
    /** @brief spin. */
    e_ControllerSpin = 1,
    /** @brief pan. */
    e_ControllerPan = 2,
    /** @brief zoom. */
    e_ControllerZoom = 3,
    /** @brief walk. */
    e_ControllerWalk = 4,
    /** @brief fly. */
    e_ControllerFly	= 5
  } PDF3DController;
  
  /**
  * @brief Enumeration for the 3D render mode.
  *
  * @details Values of this enumeration should be used alone.
  */
  typedef enum _PDF3DRenderMode {
    /** @brief Solid. */
    e_RenderModeSolid = 1,
    /** @brief Solid wireframe. */
    e_RenderModeSolidWireframe = 2,
    /** @brief Solid outline. */
    e_RenderModeSolidOutline = 3,
    /** @brief Transparent. */
    e_RenderModeTransparent = 4,
    /** @brief Transparent wireframe. */
    e_RenderModeTransparentWireframe = 5,
    /** @brief Bounding box. */
    e_RenderModeBoundingBox = 6,
    /** @brief Transparent bounding box. */
    e_RenderModeTransparentBoundingBox = 7,
    /** @brief Transparent bounding box outline. */
    e_RenderModeTransparentBoundingBoxOutline = 8,
    /** @brief Wireframe. */
    e_RenderModeWireframe = 9,
    /** @brief Shaded wireframe. */
    e_RenderModeShadedWireframe = 10,
    /** @brief Hidden wireframe. */
    e_RenderModeHiddenWireframe = 11,
    /** @brief Illustration. */
    e_RenderModeIllustration = 12,
    /** @brief Shaded illustration. */
    e_RenderModeShadedIllustration = 13,
    /** @brief Vertices. */
    e_RenderModeVertices = 14,
    /** @brief Shaded vertices. */
    e_RenderModeShadedVertices = 15
  } PDF3DRenderMode;
  

  /** @brief Constructor. */
  PDF3DAnnotInstance(){}

  /**
   * @brief Constructor, with another 3d annot instance object.
   *
   * @param[in] other  Another 3d annot instance object.
   */
  PDF3DAnnotInstance(const PDF3DAnnotInstance& other);

  /** @brief Destructor. */
  ~PDF3DAnnotInstance();

  /**
   * @brief Assign operator.
   *
   * @param[in] other  Another 3d annot instance object, whose value would be assigned to current object.
   *
   * @return Reference to current object itself.
   */
  PDF3DAnnotInstance& operator=(const PDF3DAnnotInstance& other);

  /**
   * @brief Equal operator.
   *
   * @param[in] other  Another 3d annot instance object. This function will check if current object is equal to this one.
   *
   * @return <b>true</b> means equal, while <b>false</b> means not equal.
   */
  bool operator==(const PDF3DAnnotInstance& other) const;

  /**
   * @brief Not equal operator.
   *
   * @param[in] other  Another 3d annot instance object. This function will check if current object is not equal to this one.
   *
   * @return <b>true</b> means not equal, while <b>false</b> means equal.
   */
  bool operator!=(const PDF3DAnnotInstance& other) const;

  /**
   * @brief Check whether current object is empty or not.
   *
   * @details When the current object is empty, that means current object is useless.
   *
   * @return <b>true</b> means current object is empty, while <b>false</b> means not.
   */
  bool IsEmpty() const;

  /**
   * @brief Get associated annotation.
   *
   * @return An annotation object.
   */
  pdf::annots::Annot GetAnnot();

  /**
   * @brief Set controller.
   *
   * @details Ensure the function @link ActivateCanvas @endlink has been called.
   *
   * @param[in] tool_tag  The controller tool to be set. Please refer to values starting from
   *                      @link PDF3DAnnotInstance::e_ControllerRotate @endlink and this should be one of these values.<br>
   *                      Default value: @link PDF3DAnnotInstance::e_ControllerRotate @endlink.
   *
   * @return None.
   */
  void SetController(PDF3DController tool_tag);

  /**
   * @brief Set model render mode.
   *
   * @details Ensure the function @link ActivateCanvas @endlink has been called.
   * 
   * @param[in] render_mode  The render mode to be set. Please refer to values starting from
   *                         @link PDF3DAnnotInstance::e_RenderModeSolid @endlink and this should be one of these values.<br>
   *                         Default value: @link PDF3DAnnotInstance::e_RenderModeSolidOutline @endlink.
   * 
   * @return None.
   */
  void SetRenderMode(PDF3DRenderMode render_mode);

  /**
   * @brief Activate canvas.
   * 
   * @param[in] screen_canvas_parent  The parent window handle used to activate the canvas.
   * 
   * @return The canvas handle.
   */
  FX_HWND ActivateCanvas(FX_HWND screen_canvas_parent);

  /**
   * @brief Update the canvas. 
   * 
   * @details Ensure the function @link ActivateCanvas @endlink has been called.
   *          Call this function when the canvas size and position need to be updated.
   * 
   * @param[in] rect  The new area of canvas in device coordinate system.
   * 
   * @return None.
   */
  void UpdateCanvas(RectI rect);

  /**
   * @brief Call this function when pressing the left button.
   * 
   * @details Ensure the function @link ActivateCanvas @endlink has been called.
   * 
   * @param[in] flag   The flag of the mouse event.
   * @param[in] point  The point of the mouse.
   * 
   * @return None.
   */
  void OnLButtonDown(int flag, PointF point);

  /**
   * @brief Call this function when releasing the left button.
   * 
   * @details Ensure the function @link ActivateCanvas @endlink has been called.
   * 
   * @param[in] flag   The flag of the mouse event.
   * @param[in] point  The point of the mouse.
   * 
   * @return None.
   */
  void OnLButtonUp(int flag, PointF point);

  /**
   * @brief Call this function when moving the mouse.
   *
   * @details Ensure the function @link ActivateCanvas @endlink has been called.
   * 
   * @param[in] flag   The flag of the mouse event.
   * @param[in] point  The point of the mouse.
   * 
   * @return None.
   */
  void OnMouseMove(int flag, PointF point);

  /**
   * @brief Call this function when scrolling the mouse wheel.
   * 
   * @details Ensure the function @link ActivateCanvas @endlink has been called.
   * 
   * @param[in] flag   The flag of the mouse event.
   * @param[in] delta  The delta of the mouse wheel.
   * @param[in] point  The point of the mouse.
   * 
   * @return None.
   */
  void OnMouseWheel(int flag, int delta, PointF point);

  /**
   * @brief Call this function when pressing the right button.
   * 
   * @details Ensure the function @link ActivateCanvas @endlink has been called.
   * 
   * @param[in] flag   The flag of the mouse event.
   * @param[in] point  The point of the mouse.
   * 
   * @return None.
   */
  void OnRButtonDown(int flag, PointF point);

  /**
   * @brief Call this function when releasing the right button.
   * 
   * @details Ensure the function @link ActivateCanvas @endlink has been called.
   * 
   * @param[in] flag   The flag of the mouse event.
   * @param[in] point  The point of the mouse.
   * 
   * @return None.
   */
  void OnRButtonUp(int flag, PointF point);

  // User is strongly recommended NOT to use this method; otherwise unknown situation may occur.
  explicit PDF3DAnnotInstance(FS_HANDLE handle);
};
/** @brief A typedef of array of @link PDF3DAnnotInstance @endlink. */
FSDK_DEFINE_ARRAY(PDF3DAnnotInstanceArray, PDF3DAnnotInstance)

/**
 * This class represents a 3D PDF context. Before using any class or methods in this module, please ensure
 * Foxit PDF SDK has been initialized successfully by function @link common::Library::Initialize @endlink
 * with a key including "3D" module.<br>
 * Currently, 3D module only supports Windows.
 * 
 * @see @link common::Library @endlink
 */
class PDF3DContext FS_FINAL : public Base{
 public:
  /** @brief Constructor, with a PDF document object.*/
  explicit PDF3DContext(const pdf::PDFDoc& doc);

  /**
   * @brief Constructor, with another 3d context object.
   *
   * @param[in] other  Another 3d context object.
   */
  PDF3DContext(const PDF3DContext& other);

  /** @brief Destructor. */
  ~PDF3DContext();

  /**
   * @brief Assign operator.
   *
   * @param[in] other  Another 3d context object, whose value would be assigned to current object.
   *
   * @return Reference to current object itself.
   */
  PDF3DContext& operator=(const PDF3DContext& other);

  /**
   * @brief Equal operator.
   *
   * @param[in] other  Another 3d context object. This function will check if current object is equal to this one.
   *
   * @return <b>true</b> means equal, while <b>false</b> means not equal.
   */
  bool operator==(const PDF3DContext& other) const;

  /**
  * @brief Not equal operator.
  *
  * @param[in] other  Another 3d context object. This function will check if current object is not equal to this one.
  *
  * @return <b>true</b> means not equal, while <b>false</b> means equal.
  */
  bool operator!=(const PDF3DContext& other) const;

  /**
   * @brief Check whether current object is empty or not.
   *
   * @details When the current object is empty, that means current object is useless.
   *
   * @return <b>true</b> means current object is empty, while <b>false</b> means not.
   */
  bool IsEmpty() const;

  /**
   * @brief Get 3d annot instance objects array by page index.
   *
   * @param[in] page_index  Page index. Valid range: from 0 to (<i>page_count</i>-1).
   *                        <i>page_count</i> is returned by function @link pdf::PDFDoc::GetPageCount @endlink.
   *
   * @return 3d annot instance objects array.
   */
  PDF3DAnnotInstanceArray GetPage3DAnnotArray(int page_index);

  /**
   * @brief Refresh 3D Objects.
   *
   * @details To dynamically display 3D effects, it is necessary to modify the interface based on loop calls.
   *
   * @return None.
   */
  void Invalidate();

  // User is strongly recommended NOT to use this method; otherwise unknown situation may occur.
  explicit PDF3DContext(FS_HANDLE handle = NULL);
};

} //pdf3d
} //addon
} //foxit
#endif //FS_3DPDF_H_
#endif //defined(_WIN32) || defined(_WIN64)
