/**
 * Copyright (C) 2003-2022, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 *
 * @file fs_pdf2office.h
 * @brief Header file for PDF2Office related definitions and classes.
 */
#ifndef FS_PDF2OFFICE_H_
#define FS_PDF2OFFICE_H_
#if defined(_WIN32) || defined(_WIN64) || (defined(__linux__) && !defined(__ANDROID__))
#include "common/fs_common.h"

/**
* @brief Foxit namespace.
*
* @details All the definitions and classes of Foxit PDF SDK are defined within this namespace.
*/
namespace foxit {
/**
 * @brief Add-on namespace.
 */
namespace addon {
/**
 * @brief Conversion namespace.
 */
namespace conversion {
/**
 * @brief pdf2office namespace.
 */
namespace pdf2office {
/** This class represents setting data used for converting PDF documents to Word format documents. */
class PDF2WordSettingData FS_FINAL : public Object{
 public:
  /** @brief Constructor. */
  PDF2WordSettingData()
    : enable_retain_page_layout(false)
    , enable_generate_headers_and_footers(false)
    , enable_generate_footnotes_and_endnotes(false){ }

  /**
   * @brief Constructor, with parameters.
   *
   * @param[in] enable_retain_page_layout               A boolean value which indicates whether retain page layout for the conversion of PDF documents to Word format documents. Default value: <b>false</b>.
   *                                                    <b>true</b> means to retain page layout for the conversion of PDF documents to Word format documents.
   *                                                    <b>false</b> means not to retain page layout for the conversion of PDF documents to Word format documents.
   * @param[in] enable_generate_headers_and_footers     A boolean value which indicates whether convert the recognized headers and footers from PDF documents to headers and footers of Word format documents
   *                                                    during the conversion of PDF documents to Word format documents. Default value: <b>false</b>.
   *                                                    <b>true</b> means to convert the recognized headers and footers from PDF documents to headers and footers of Word format documents.
   *                                                    <b>false</b> means to convert the recognized headers and footers from PDF documents to the main texts or text boxes of Word format documents.
   * @param[in] enable_generate_footnotes_and_endnotes  A boolean value which indicates whether convert the recognized footnotes and endnotes from PDF documents to footnotes and endnotes of Word format documents
   *                                                    during the conversion of PDF documents to Word format documents. Default value: <b>false</b>.
   *                                                    <b>true</b> means to convert the recognized footnotes and endnotes from PDF documents to footnotes and endnotes of Word format documents.
   *                                                    <b>false</b> means to convert the recognized footnotes and endnotes from PDF documents to the main texts or text boxes of Word format documents.
   *                                                    Default value:<b>false</b>.
   *
   */
  PDF2WordSettingData(bool enable_retain_page_layout, bool enable_generate_headers_and_footers, bool enable_generate_footnotes_and_endnotes) {
    this->enable_retain_page_layout = enable_retain_page_layout;
    this->enable_generate_headers_and_footers = enable_generate_headers_and_footers;
    this->enable_generate_footnotes_and_endnotes = enable_generate_footnotes_and_endnotes;
  }

  /**
   * @brief Assign operator.
   *
   * @param[in] data  Another PDF-to-Word setting data object, whose value would be assigned to current object.
   *
   * @return Reference to current object itself.
   */
  PDF2WordSettingData & operator = (const PDF2WordSettingData & data) {
    enable_retain_page_layout = data.enable_retain_page_layout;
    enable_generate_headers_and_footers = data.enable_generate_headers_and_footers;
    enable_generate_footnotes_and_endnotes = data.enable_generate_footnotes_and_endnotes;
    return (*this);
  }

  /**
   * @brief Set value.
   *
   * @param[in] enable_retain_page_layout               A boolean value which indicates whether retain page layout for the conversion of PDF documents to Word format documents. Default value: <b>false</b>.
   *                                                    <b>true</b> means to retain page layout for the conversion of PDF documents to Word format documents.
   *                                                    <b>false</b> means not to retain page layout for the conversion of PDF documents to Word format documents.
   * @param[in] enable_generate_headers_and_footers     A boolean value which indicates whether convert the recognized headers and footers from PDF documents to headers and footers of Word format documents
   *                                                    during the conversion of PDF documents to Word format documents. Default value: <b>false</b>.
   *                                                    <b>true</b> means to convert the recognized headers and footers from PDF documents to headers and footers of Word format documents.
   *                                                    <b>false</b> means to convert the recognized headers and footers from PDF documents to the main texts or text boxes of Word format documents.
   * @param[in] enable_generate_footnotes_and_endnotes  A boolean value which indicates whether convert the recognized footnotes and endnotes from PDF documents to footnotes and endnotes of Word format documents
   *                                                    during the conversion of PDF documents to Word format documents. Default value: <b>false</b>.
   *                                                    <b>true</b> means to convert the recognized footnotes and endnotes from PDF documents to footnotes and endnotes of Word format documents.
   *                                                    <b>false</b> means to convert the recognized footnotes and endnotes from PDF documents to the main texts or text boxes of Word format documents.
   *                                                    Default value:<b>false</b>.
   *
   * @return None.
   */
  void Set(bool enable_retain_page_layout, bool enable_generate_headers_and_footers, bool enable_generate_footnotes_and_endnotes) {
    this->enable_retain_page_layout = enable_retain_page_layout;
    this->enable_generate_headers_and_footers = enable_generate_headers_and_footers;
    this->enable_generate_footnotes_and_endnotes = enable_generate_footnotes_and_endnotes;
  }

  /**
   * @brief A boolean value which indicates whether retain page layout for the conversion of PDF documents to Word format documents. Default value: <b>false</b>.
   *
   * @details <b>true</b> means to retain page layout for the conversion of PDF documents to Word format documents.
   *          <b>false</b> means not to retain page layout for the conversion of PDF documents to Word format documents.
   */
  bool enable_retain_page_layout;

  /**
   * @brief A boolean value which indicates whether convert the recognized headers and footers from PDF documents to headers and footers
   *        of word format documents during the conversion of PDF documents to word format documents. Default value: <b>false</b>.
   *
   * @details <b>true</b> means to convert the recognized headers and footers from PDF documents to headers and footers of word format documents.
   *          <b>false</b> means to convert the recognized headers and footers from PDF documents to the main texts or text boxes of word format documents.
   */
  bool enable_generate_headers_and_footers;

  /**
   * @brief A boolean value which indicates whether convert the recognized footnotes and endnotes from PDF documents to footnotes and endnotes
   *        of word format documents during the conversion of PDF documents to word format documents. Default value: <b>false</b>.
   *
   * @details <b>true</b> means to convert the recognized footnotes and endnotes from PDF documents to footnotes and endnotes of word format documents.
   *          <b>false</b> means to convert the recognized footnotes and endnotes from PDF documents to the main texts or text boxes of word format documents.
   */
  bool enable_generate_footnotes_and_endnotes;
};

/** This class represents setting data used for converting PDF documents to Power Point format documents. */
class PDF2PowerPointSettingData FS_FINAL : public Object{
 public:
  /** @brief Constructor. */
  PDF2PowerPointSettingData()
    : enable_aggressively_split_sections(false) {}

  /**
   * @brief Constructor, with parameters.
   *
   * @param[in] enable_aggressively_split_sections  A boolean value which indicates whether split sections aggressively for the conversion of PDF documents to Power Point format documents. Default value: <b>false</b>.
   *                                                In two paragraphs of the same text box, if the spacing between them is large enough, they will be split into two separate text boxes 
   *                                                for the conversion of PDF documents to Power Point format documents.
   *                                                <b>true</b> means to split sections aggressively for the conversion of PDF documents to Power Point format documents.
   *                                                <b>false</b> means not to split sections aggressively for the conversion of PDF documents to Power Point format documents.
   */
  PDF2PowerPointSettingData(bool enable_aggressively_split_sections) {
    this->enable_aggressively_split_sections = enable_aggressively_split_sections;
  }

  /**
   * @brief Assign operator.
   *
   * @param[in] data  Another PDF-to-Power-Point setting data object, whose value would be assigned to current object.
   *
   * @return Reference to current object itself.
   */
  PDF2PowerPointSettingData & operator = (const PDF2PowerPointSettingData & data) {
    enable_aggressively_split_sections = data.enable_aggressively_split_sections;
    return (*this);
  }

  /**
   * @brief Set value.
   *
   * @param[in] enable_aggressively_split_sections  A boolean value which indicates whether split sections aggressively for the conversion of PDF documents to Power Point format documents. Default value: <b>false</b>.
   *                                                In two paragraphs of the same text box, if the spacing between them is large enough, they will be split into two separate text boxes 
   *                                                for the conversion of PDF documents to Power Point format documents.
   *                                                <b>true</b> means to split sections aggressively for the conversion of PDF documents to Power Point format documents.
   *                                                <b>false</b> means not to split sections aggressively for the conversion of PDF documents to Power Point format documents.
   *
   * @return None.
   */
  void Set(bool enable_aggressively_split_sections) {
    this->enable_aggressively_split_sections = enable_aggressively_split_sections;
  }

  /**
   * @brief A boolean value which indicates whether split sections aggressively for the conversion of PDF documents to Power Point format documents. Default value: <b>false</b>.
   *
   * @details In two paragraphs of the same text box, if the spacing between them is large enough, they will be split into two separate text boxes 
   *          for the conversion of PDF documents to Power Point format documents.
   *          <b>true</b> means to split sections aggressively for the conversion of PDF documents to Power Point format documents.
   *          <b>false</b> means not to split sections aggressively for the conversion of PDF documents to Power Point format documents.
   */
  bool enable_aggressively_split_sections;
};

/** This class represents setting data used for converting PDF documents to Excel format documents. */
class PDF2ExcelSettingData FS_FINAL : public Object{
 public:
  /** @brief Constructor. */
  PDF2ExcelSettingData()
    : decimal_symbol("")
    , thousands_separator("") { }

  /**
   * @brief Constructor, with parameters.
   *
   * @param[in] decimal_symbol       A string value which indicates being recognized as decimal during the conversion of PDF documents to excel format documents.
   * @param[in] thousands_separator  A string value which indicates being recognized as thousands separator during the conversion of PDF documents to excel format documents.
   */
  PDF2ExcelSettingData(String decimal_symbol, String thousands_separator) {
    this->decimal_symbol = decimal_symbol;
    this->thousands_separator = thousands_separator;
  }

  /**
   * @brief Assign operator.
   *
   * @param[in] data  Another PDF-to-Excel setting data object, whose value would be assigned to current object.
   *
   * @return Reference to current object itself.
   */
  PDF2ExcelSettingData & operator = (const PDF2ExcelSettingData & data) {
    this->decimal_symbol = data.decimal_symbol;
    this->thousands_separator = data.thousands_separator;
    return (*this);
  }

  /**
   * @brief Set value.
   *
   * @param[in] decimal_symbol       A string value which indicates being recognized as decimal during the conversion of PDF documents to excel format documents.
   * @param[in] thousands_separator  A string value which indicates being recognized as thousands separator during the conversion of PDF documents to excel format documents.
   *
   * @return None.
   */
  void Set(String decimal_symbol, String thousands_separator) {
    this->decimal_symbol = decimal_symbol;
    this->thousands_separator = thousands_separator;
  }

  /**
   * @brief A string value which indicates being recognized as decimal during the conversion of PDF documents to excel format documents.
   *
   * @details The decimal symbol is used to separate the integer part and the decimal part of a number.
   *          If this parameter is set to ".", then the number "123.456" will be recognized as 123.456.
   *          If this parameter is set to ",", then the number "123,456" will be recognized as 123.456.
   *          If this parameter is not set, the decimal symbol will be recognized automatically.
   *
   * @note Currently, this parameter is only useful for cells with only numbers and separators.
   *       The value of this parameter must be different from the value of parameter <i>thousands_separator</i>.
   *       To ensure the correct recognition of the decimal symbol, this parameter should be set together
   *       with parameter <i>thousands_separator</i>.
   */
  String decimal_symbol;

  /**
   * @brief A string value which indicates being recognized as thousands separator during the conversion of PDF documents to excel format documents.
   *
   * @details The thousands separator is used to separate the thousands part of a number.
   *          If this parameter is set to ",", then the number "123,456" will be recognized as 123456.
   *          If this parameter is set to ".", then the number "123.456" will be recognized as 123456.
   *          If this parameter is set to " ", then the number "123 456" will be recognized as 123456.
   *          If this parameter is not set, the thousands separator will be recognized automatically.
   *
   * @note Currently, this parameter is only useful for cells with only numbers and separators.
   *       The value of this parameter must be different from the value of parameter <i>decimal_symbol</i>.
   *       To ensure the correct recognition of the thousands separator, this parameter should be set together
   *       with parameter <i>decimal_symbol</i>.
   */
  String thousands_separator;

};

/** This class represents setting data used for converting PDF to Office(Word, Excel or PowerPoint) format file. */
class PDF2OfficeSettingData FS_FINAL : public Object {
 public:
  /** @brief Constructor. */
  PDF2OfficeSettingData()
    : enable_ml_recognition(false)
    , include_pdf_comments(true) {}

  /**
   * @brief Constructor, with parameters.
   *
   * @param[in] metrics_data_folder_path  A valid path of a folder which contains metrics data files. This should not be an empty string. 
   *                                      These metrics data files are used to simulate the office format document typesetting process during conversion.
   *                                      They are offered in the "res/metrics_data" folder of the Foxit PDF Conversion SDK package.
   * @param[in] enable_ml_recognition     A boolean value which indicates whether enable machine learning-based recognition functionality. Default value: <b>false</b>.
   *                                      <b>true</b> means enable machine learning-based recognition functionality to identify borderless tables in PDF documents. 
   *                                      In order to convert the tables better, it will draw a black border with a width of 1 on the recognized borderless table.
   *                                      This will be improved in subsequent versions.
   *                                      And this recognition functionality will be executed on the server side and return the relevant results when it is done. 
   *                                      <b>false</b> means disable machine learning-based recognition functionality.
   *                                      And the recognition functionality not based on machine learning will be enabled.
   * @param[in] page_range                A range object that specifies some pages. These pages will be used to convert PDF document to office format documents. Default value: <b>true/b>.
   * @param[in] include_pdf_comments      A boolean value which indicates whether to include PDF documents comments for the conversion of PDF documents to office format documents. Default value: <b>true/b>.
   *                                      <b>true</b> means to include PDF document comments for the conversion of PDF documents to office format documents.
   *                                      <b>false</b> means not to include PDF document comments for the conversion of PDF documents to office format documents.
   * @param[in] word_setting_data         A setting data object that used for converting PDF documents to Word format documents.
   * @param[in] power_point_setting_data  A setting data object that used for converting PDF documents to Power Point format documents.
   * @param[in] excel_setting_data        A setting data object that used for converting PDF documents to Excel format documents.
   *
   * @note Our machine learning-based technology for identifying borderless tables uses HTTPS and requires a network connection to send the images of the rendered PDF pages to our server during the conversion process.
   */
  PDF2OfficeSettingData(const wchar_t* metrics_data_folder_path, bool enable_ml_recognition, const common::Range& page_range, bool include_pdf_comments,
                const PDF2WordSettingData& word_setting_data, const PDF2PowerPointSettingData& power_point_setting_data,
                const PDF2ExcelSettingData& excel_setting_data) {
    this->metrics_data_folder_path = metrics_data_folder_path;
    this->enable_ml_recognition = enable_ml_recognition;
    this->page_range = page_range;
    this->include_pdf_comments = include_pdf_comments;
    this->word_setting_data = word_setting_data;
    this->power_point_setting_data = power_point_setting_data;
  }

  /**
   * @brief Assign operator.
   *
   * @param[in] data  Another PDF-to-Office setting data object, whose value would be assigned to current object.
   *
   * @return Reference to current object itself.
   */
  PDF2OfficeSettingData &operator = (const PDF2OfficeSettingData& data) {
    metrics_data_folder_path = data.metrics_data_folder_path;
    enable_ml_recognition = data.enable_ml_recognition;
    page_range = data.page_range;
    include_pdf_comments = data.include_pdf_comments;
    word_setting_data = data.word_setting_data;
    power_point_setting_data = data.power_point_setting_data;

    return (*this);
  }

  /**
   * @brief Set value.
   *
   * @param[in] metrics_data_folder_path  A valid path of a folder which contains metrics data files. This should not be an empty string. 
   *                                      These metrics data files are used to simulate the office format document typesetting process during conversion.
   *                                      They are offered in the "res/metrics_data" folder of the Foxit PDF Conversion SDK package.
   * @param[in] enable_ml_recognition     A boolean value which indicates whether enable machine learning-based recognition functionality. Default value: <b>false</b>.
   *                                      <b>true</b> means enable machine learning-based recognition functionality to identify borderless tables in PDF documents. 
   *                                      In order to convert the tables better, it will draw a black border with a width of 1 on the recognized borderless table.
   *                                      This will be improved in subsequent versions.
   *                                      And this recognition functionality will be executed on the server side and return the relevant results when it is done. 
   *                                      <b>false</b> means disable machine learning-based recognition functionality.
   *                                      And the recognition functionality not based on machine learning will be enabled.
   * @param[in] page_range                A range object that specifies some pages. These pages will be used to convert PDF documents to office format documents.
   * @param[in] include_pdf_comments      A boolean value which indicates whether to include PDF documents comments for the conversion of PDF documents to office format documents. Default value: <b>true/b>.
   *                                      <b>true</b> means to include PDF document comments for the conversion of PDF documents to office format documents.
   *                                      <b>false</b> means not to include PDF document comments for the conversion of PDF documents to office format documents.
   * @param[in] word_setting_data         A setting data object that used for converting PDF documents to Word format documents.
   * @param[in] power_point_setting_data  A setting data object that used for converting PDF documents to Power Point format documents.
   * @param[in] excel_setting_data        A setting data object that used for converting PDF documents to Excel format documents.
   * 
   * @note Our machine learning-based technology for identifying borderless tables uses HTTPS and requires a network connection to send the images of the rendered PDF pages to our server during the conversion process.   
   *
   * @return None.
   */
  void Set(const wchar_t* metrics_data_folder_path, bool enable_ml_recognition, const common::Range &page_range, bool include_pdf_comments,
                const PDF2WordSettingData& word_setting_data, const PDF2PowerPointSettingData& power_point_setting_data,
                const PDF2ExcelSettingData& excel_setting_data) {
    this->metrics_data_folder_path = metrics_data_folder_path;
    this->enable_ml_recognition = enable_ml_recognition;
    this->page_range = page_range;
    this->include_pdf_comments = include_pdf_comments;
    this->word_setting_data = word_setting_data;
    this->power_point_setting_data = power_point_setting_data;
  }

  /**
   * @brief A valid path of a folder which contains metrics data files. This should not be an empty string.
   *
   * @details These metrics data files are used to simulate the office format document typesetting process during conversion.
   *          They are offered in the "res/metrics_data" folder of the Foxit PDF Conversion SDK package.
   */
  WString metrics_data_folder_path;

  /**
   * @brief A boolean value which indicates whether enable machine learning-based recognition functionality. Default value: <b>false</b>.
   *
   * @details <b>true</b> means enable machine learning-based recognition functionality to identify borderless tables in PDF documents.
   *          And this recognition functionality will be executed on the server side and return the relevant results when it is done.
   *          In order to convert the tables better, it will draw a black border with a width of 1 on the recognized borderless table.
   *          This will be improved in subsequent versions.
   *          <b>false</b> means disable machine learning-based recognition functionality.
   *          And the recognition functionality not based on machine learning will be enabled.
   *
   * @note Our machine learning-based technology for identifying borderless tables uses HTTPS and requires a network connection to send the images of the rendered PDF pages to our server during the conversion process.
   */
  bool enable_ml_recognition;

  /**
   * @brief A range object that specifies some pages. These pages will be used to convert PDF documents to office format documents.
   *
   * @details The valid page range: from 0 to (<i>count</i> - 1). <i>count</i> is returned by function @link PDFDoc::GetPageCount @endlink.
   *          If this range object is constructed by default constructor and not set any value, that means all the pages of
   *          current document will be used to convert PDF documents to office format documents.
   */
  common::Range page_range;

  /**
   * @brief A boolean value which indicates whether to include PDF documents comments
   *        for the conversion of PDF documents to office format documents. Default value: <b>true/b>.
   *
   * @details <b>true</b> means to include PDF document comments for the conversion of PDF documents to office format documents.
   *          <b>false</b> means not to include PDF document comments for the conversion of PDF documents to office format documents.
   */
  bool include_pdf_comments;

  /**
   * @brief A setting data object that used for converting PDF documents to Word format documents.
   *
   * @details This setting is only vaild for the conversion of PDF documents to word format documents.
   *          For more details, please refer to class @link PDF2WordSettingData @endlink.
   */
  PDF2WordSettingData word_setting_data;

  /**
 * @brief A setting data object that used for converting PDF documents to Power Point format documents.
 *
 * @details This setting is only vaild for the conversion of PDF documents to Power Point format documents.
 *          For more details, please refer to class @link PDF2PowerPointSettingData @endlink.
 */
  PDF2PowerPointSettingData power_point_setting_data;

  /**
 * @brief A setting data object that used for converting PDF documents to Excel format documents.
 *
 * @details This setting is only vaild for the conversion of PDF documents to Excel format documents.
 *          For more details, please refer to class @link PDF2ExcelSettingData @endlink.
 */
  PDF2ExcelSettingData excel_setting_data;
};

/**
 * This class represents a callback object used to pause and notify the conversion progress during the converting process.
 * All the pure virtual functions in this class are used as callback functions and should be implemented by user.
 */
class ConvertCallback {
 public:
  /**
   * @brief A callback function used to pause the current conversion progress.
   *
   * @return <b>true</b> means to pause now, while <b>false</b> means not to pause now.
   */
  virtual bool NeedToPause() = 0;
  
  /**
   * @brief A callback function used to notify the current conversion progress.
   *
   * @param[in] converted_count  The converted page count.
   * @param[in] total_count      The total page count.
   *
   * @return None.
   */
  virtual void ProgressNotify(int converted_count, int total_count) = 0;
};

/**
 * This class can be used to convert PDF files to Office(Word, Excel or PowerPoint) format files.
 * Before using "Conversion" module, please ensure the resource folders named "res" and "lib" in the Foxit PDF Conversion SDK package is valid.
 * Before using methods in this module, please ensure Foxit PDF SDK has been initialized successfully
 * by function @link common::Library::Initialize @endlink with a key including "PDF2Office" module.
 *
 * @see @link common::Library @endlink
 */
class PDF2Office FS_FINAL : public Base {
 public:
  /**
   * @brief Initialize Foxit PDF Conversion SDK Library.
   *
   * @details During the life-cycle of "pdf2office" module, this function can only be called once and
   *          should be called first before any other functions in "pdf2office" module can be called.
   *
   * @param[in] library_path  Path of Foxit PDF Conversion SDK library. This should not be an empty string.
   *
   * @return None.
   */
  static void Initialize(const wchar_t* library_path);

  /**
   * @brief Release all resource allocated by Foxit PDF Conversion SDK Library.
   *
   * @details When the user does not use "pdf2office" module, this function should be called to release all memory blocks allocated by the library.
   *
   * @return None.
   */
  static void Release();

  /**
   * @brief Start to convert a PDF file to a Word format file. Function @link PDF2Office::Initialize @endlink must be called 
   *        before calling this function.
   *
   * @details Currently only support converting to DOCX format file.
   *
   * @param[in] src_pdf_path          Path of a PDF file. This should not be an empty string.
   * @param[in] src_pdf_password      Password for the input PDF file. If no password is needed for the file, please pass an empty string.
   * @param[in] saved_word_file_path  Path of the saved Word format file as conversion result. This should not be an empty string.
   *                                  If the suffix name of the saved Word format file is not "docx", a new suffix named "docx" will be added to the original file name.
   * @param[in] setting_data          Setting data used for converting.If the parameter <i>page_range</i> of @link PDF2OfficeSettingData @endlink 
   *                                  object is empty, that means convert each page of the PDF file to Word format file.
   * @param[in] convert_callback      A @link conversion::pdf2office::ConvertCallback @endlink object which is implemented by user
   *                                  to pause and notify the conversion progress during the converting process.
   *                                  This can be <b>NULL</b> which means not to pause and notify the conversion progress.
   *                                  If this is not <b>NULL</b>, it should be a valid @link conversion::pdf2office::ConvertCallback @endlink 
   *                                  object implemented by user.<br>
   *                                  Default value: <b>NULL</b>.
   *
   * @return A progressive object. Please check the rate of current progress by function
   *         @link common::Progressive::GetRateOfProgress @endlink. If the rate is not 100 yet, call function
   *         @link common::Progressive::Continue @endlink to continue the progress until the progress is finished.
   */
  static common::Progressive StartConvertToWord(const wchar_t* src_pdf_path, const wchar_t* src_pdf_password, const wchar_t* saved_word_file_path, 
                            const PDF2OfficeSettingData& setting_data, ConvertCallback* convert_callback = NULL);
   
  /**
   * @brief Start to convert a PDF file to a Word format file. Function @link PDF2Office::Initialize @endlink must be called 
   *        before calling this function.
   *
   * @details Currently only support converting to DOCX format file.
   *
   * @param[in] src_pdf_reader          A @link ReaderCallback @endlink object which is implemented by user to
   *                                    load a PDF document. It should not be <b>NULL</b>.
   * @param[in] src_pdf_password        Password for the input PDF file. If no password is needed for the file, please pass an empty string.
   * @param[in] saved_word_file_stream  A @link foxit::common::file::StreamCallback @endlink object which is implemented by user 
   *                                    to read the contents of the converted Word format file. It should not be <b>NULL</b>.
   * @param[in] setting_data            Setting data used for converting.If the parameter <i>page_range</i> of @link PDF2OfficeSettingData @endlink 
   *                                    object is empty, that means convert each page of the PDF file to Word format file.
   * @param[in] convert_callback        A @link conversion::pdf2office::ConvertCallback @endlink object which is implemented by user
   *                                    to pause and notify the conversion progress during the converting process.
   *                                    This can be <b>NULL</b> which means not to pause and notify the conversion progress.
   *                                    If this is not <b>NULL</b>, it should be a valid @link conversion::pdf2office::ConvertCallback @endlink 
   *                                    object implemented by user.<br>
   *                                    Default value: <b>NULL</b>.
   *
   * @return A progressive object. Please check the rate of current progress by function
   *         @link common::Progressive::GetRateOfProgress @endlink. If the rate is not 100 yet, call function
   *         @link common::Progressive::Continue @endlink to continue the progress until the progress is finished..
   */
  static common::Progressive StartConvertToWord(common::file::ReaderCallback* src_pdf_reader, const wchar_t* src_pdf_password, common::file::StreamCallback* saved_word_file_stream, 
                            const PDF2OfficeSettingData& setting_data, ConvertCallback* convert_callback = NULL);
  
  /**
   * @brief Start to convert a PDF file to a Excel format file. Function @link PDF2Office::Initialize @endlink must be called 
   *        before calling this function.
   *
   * @details Currently only support converting to XLSX format file.
   *
   * @param[in] src_pdf_path           Path of a PDF file. This should not be an empty string.
   * @param[in] src_pdf_password       Password for the input PDF file. If no password is needed for the file, please pass an empty string.
   * @param[in] saved_excel_file_path  Path of the saved Excel format file as conversion result. This should not be an empty string.
   *                                   If the suffix name of the saved Excel format file is not "xlsx", a new suffix named "xlsx" will be added to the original file name.
   * @param[in] setting_data           Setting data used for converting.If the parameter <i>page_range</i> of @link PDF2OfficeSettingData @endlink 
   *                                   object is empty, that means convert each page of the PDF file to Excel format file.
   * @param[in] convert_callback       A @link conversion::pdf2office::ConvertCallback @endlink object which is implemented by user
   *                                   to pause and notify the conversion progress during the converting process.
   *                                   This can be <b>NULL</b> which means not to pause and notify the conversion progress.
   *                                   If this is not <b>NULL</b>, it should be a valid @link conversion::pdf2office::ConvertCallback @endlink 
   *                                   object implemented by user.<br>
   *                                   Default value: <b>NULL</b>.
   *
   * @return A progressive object. Please check the rate of current progress by function
   *         @link common::Progressive::GetRateOfProgress @endlink. If the rate is not 100 yet, call function
   *         @link common::Progressive::Continue @endlink to continue the progress until the progress is finished.
   */
  static common::Progressive StartConvertToExcel(const wchar_t* src_pdf_path, const wchar_t* src_pdf_password, const wchar_t* saved_excel_file_path, 
                             const PDF2OfficeSettingData& setting_data, ConvertCallback* convert_callback = NULL);
   
  /**
   * @brief Start to convert a PDF file to a Excel format file. Function @link PDF2Office::Initialize @endlink must be called 
   *        before calling this function.
   *
   * @details Currently only support converting to XLSX format file.
   *
   * @param[in] src_pdf_reader           A @link ReaderCallback @endlink object which is implemented by user to
   *                                     load a PDF document. It should not be <b>NULL</b>.
   * @param[in] src_pdf_password         Password for the input PDF file. If no password is needed for the file, please pass an empty string.
   * @param[in] saved_excel_file_stream  A @link foxit::common::file::StreamCallback @endlink object which is implemented by user 
   *                                     to read the contents of the converted Excel format file. It should not be <b>NULL</b>.
   * @param[in] setting_data             Setting data used for converting.If the parameter <i>page_range</i> of @link PDF2OfficeSettingData @endlink 
   *                                     object is empty, that means convert each page of the PDF file to Excel format file.
   * @param[in] convert_callback         A @link conversion::pdf2office::ConvertCallback @endlink object which is implemented by user
   *                                     to pause and notify the conversion progress during the converting process.
   *                                     This can be <b>NULL</b> which means not to pause and notify the conversion progress.
   *                                     If this is not <b>NULL</b>, it should be a valid @link conversion::pdf2office::ConvertCallback @endlink 
   *                                     object implemented by user.<br>
   *                                     Default value: <b>NULL</b>.
   *
   * @return A progressive object. Please check the rate of current progress by function
   *         @link common::Progressive::GetRateOfProgress @endlink. If the rate is not 100 yet, call function
   *         @link common::Progressive::Continue @endlink to continue the progress until the progress is finished.
   */
  static common::Progressive StartConvertToExcel(common::file::ReaderCallback* src_pdf_reader, const wchar_t* src_pdf_password, common::file::StreamCallback* saved_excel_file_stream, 
                             const PDF2OfficeSettingData& setting_data, ConvertCallback* convert_callback = NULL);
  
  /**
   * @brief Start to convert a PDF file to a PowerPoint format file. Function @link PDF2Office::Initialize @endlink must be called 
   *        before calling this function.
   *
   * @details Currently only support converting to PPTX format file.
   *
   * @param[in] src_pdf_path         Path of a PDF file. This should not be an empty string.
   * @param[in] src_pdf_password     Password for the input PDF file. If no password is needed for the file, please pass an empty string.
   * @param[in] saved_ppt_file_path  Path of the saved PowerPoint format file as conversion result. This should not be an empty string.
   *                                 If the suffix name of the saved PowerPoint format file is not "pptx", a new suffix named "pptx" will be added to the original file name.
   * @param[in] setting_data         Setting data used for converting.If the parameter <i>page_range</i> of @link PDF2OfficeSettingData @endlink 
   *                                 object is empty, that means convert each page of the PDF file to Power Point format file.
   * @param[in] convert_callback     A @link conversion::pdf2office::ConvertCallback @endlink object which is implemented by user
   *                                 to pause and notify the conversion progress during the converting process.
   *                                 This can be <b>NULL</b> which means not to pause and notify the conversion progress.
   *                                 If this is not <b>NULL</b>, it should be a valid @link conversion::pdf2office::ConvertCallback @endlink 
   *                                 object implemented by user.<br>
   *                                 Default value: <b>NULL</b>.
   *
   * @return A progressive object. Please check the rate of current progress by function
   *         @link common::Progressive::GetRateOfProgress @endlink. If the rate is not 100 yet, call function
   *         @link common::Progressive::Continue @endlink to continue the progress until the progress is finished.
   */
  static common::Progressive StartConvertToPowerPoint(const wchar_t* src_pdf_path, const wchar_t* src_pdf_password, const wchar_t* saved_ppt_file_path, 
                                  const PDF2OfficeSettingData& setting_data, ConvertCallback* convert_callback = NULL);
  
  /**
   * @brief Start to convert a PDF file to a PowerPoint format file. Function @link PDF2Office::Initialize @endlink must be called 
   *        before calling this function.
   *
   * @details Currently only support converting to PPTX format file.
   *
   * @param[in] src_pdf_reader         A @link ReaderCallback @endlink object which is implemented by user to
   *                                   load a PDF document. It should not be <b>NULL</b>.
   * @param[in] src_pdf_password       Password for the input PDF file. If no password is needed for the file, please pass an empty string.
   * @param[in] saved_ppt_file_stream  A @link foxit::common::file::StreamCallback @endlink object which is implemented by user 
   *                                   to read the contents of the converted PowerPoint format file. It should not be <b>NULL</b>.
   * @param[in] setting_data           Setting data used for converting.If the parameter <i>page_range</i> of @link PDF2OfficeSettingData @endlink 
   *                                   object is empty, that means convert each page of the PDF file to Power Point format file.
   * @param[in] convert_callback       A @link conversion::pdf2office::ConvertCallback @endlink object which is implemented by user
   *                                   to pause and notify the conversion progress during the converting process.
   *                                   This can be <b>NULL</b> which means not to pause and notify the conversion progress.
   *                                   If this is not <b>NULL</b>, it should be a valid @link conversion::pdf2office::ConvertCallback @endlink 
   *                                   object implemented by user.<br>
   *                                   Default value: <b>NULL</b>.
   *
   * @return A progressive object. Please check the rate of current progress by function
   *         @link common::Progressive::GetRateOfProgress @endlink. If the rate is not 100 yet, call function
   *         @link common::Progressive::Continue @endlink to continue the progress until the progress is finished.
   */
  static common::Progressive StartConvertToPowerPoint(common::file::ReaderCallback* src_pdf_reader, const wchar_t* src_pdf_password, common::file::StreamCallback* saved_ppt_file_stream, 
                                  const PDF2OfficeSettingData& setting_data, ConvertCallback* convert_callback = NULL);

};

}  // namespace pdf2office
}  // namespace conversion
}  // namespace addon
}  // namespace foxit

#endif //defined(WIN32) 
#endif
